#include <stdio.h>
#include <iostream>

#include "ko/IntersectionDetector.hpp"

using namespace std;

namespace ko
{

IntersectionDetector::IntersectionDetector()
{
  spacetree1_pool = new list<BSPTree *>();
  spacetree1	  = new BSPTree( spacetree1_pool );
}

void IntersectionDetector::init(list<Element *> &targets)
{
  elements.clear();
  elements = targets;

  buildTrees();
}
 


void IntersectionDetector::buildTrees()
{
  BSPTree::pool_free_bsptree( spacetree1, spacetree1_pool );

  spacetree1 = BSPTree::pool_alloc_bsptree( spacetree1_pool );

  Vector min(3), max(3), center(3);

  list<Vector *> positions;

  for( list<Element *>::iterator it = elements.begin();
       it != elements.end(); it++)
  {
    Element &e = **it;

    positions.push_back(&(e.position));
  }


  bbox(positions, min, max);

  center = min;
  center += max;
  center *= 0.5;

  spacetree1->min = min;
  spacetree1->max = max;
  spacetree1->center = center;

  list<Element *> colliders = elements;

  spacetree1->Partition(colliders);
}

void IntersectionDetector::intersectionList(Element &e1, list<Element *> &intersections)
{
//   std::cout << "ilist: " << e1.index << endl;

  list<Element *> potentials;

  spacetree1->Collide(e1, potentials);

  for(list<Element *>::iterator it = potentials.begin();
      it != potentials.end(); it++)
  {
    Element &e2 = **it;

//     std::cout << "potentials: " << e2.index << endl;
      
    if(&e1 != &e2 && !e1.isConnected(e2) && isIntersection(e1, e2))
    {
      intersections.push_back(&e2);
    }
  }
}


bool IntersectionDetector::collisions(Physical& physical)
{
  //buildTrees();
  init(physical.elements);
  
  for(list<Element *>::iterator it = physical.elements.begin();
      it != physical.elements.end(); it++)
  {
    Element &e1 = **it;
      
    list<Element *> collisions;

    intersectionList(e1, collisions);

    for(list<Element *>::iterator it2 = collisions.begin();
	it2 != collisions.end(); it2++)
    {
      Element &e2 = **it2;

      if(isIntersection(e1, e2))
      {
// 	cerr << "collision!" << endl;
	
	Connection &c = *(new Connection());
	
	c.connect(e1, e2);
	c.transient = true;
	
	Vector r(3);
	
	r = e2.position;
	r.axpy(-1, e1.position);
	
	c.length = e1.radius + e2.radius;
	
	physical.addConnection(c);

// 	cout << "contact connection: " << endl;

// 	cout << "e1: " << e1.name << endl;
// 	cout << "e2: " << e2.name << endl;
	
// 	cout << "hooke: " << MechanicsModel::hooke(c) << endl;
// 	cout << "damp: " << MechanicsModel::damp(c) << endl;


	// Check error

	Vector vrel(3);
    
	vrel = e1.velocity;
	vrel.axpy(-1, e2.velocity);
	
// 	if(vrel.norm() * stepsize > min(e1.radius, e2.radius) / 1)
// 	{
// 	  cerr << "large error collision predicted: " <<
// 	    vrel.norm() * stepsize / min(e1.radius, e2.radius) << endl;
// 	  return false;

// 	}
      }
    }
  }

  return true;
}

void IntersectionDetector::fractures(Physical& physical)
{
  list<Connection *> removals;

  for(list<Connection *>::iterator it = physical.connections.begin();
      it != physical.connections.end(); it++)
  {
    Connection &c = **it;

    double strength;

    if(c.transient)
    {
//       cerr << "transient" << endl;
      strength = (c.element1->material->collisionStrength +
		  c.element2->material->collisionStrength) / 2 / c.damage;
    }
    else
    {
      strength = (c.element1->material->strength +
		  c.element2->material->strength) / 2 /	c.damage;
    }

    //cerr << "tension: " << c.tension << endl;
    //cerr << c.element1->name << " <-> " << c.element2->name << endl;

    if(c.tension > strength)
    {
//       cerr << "fracture: " << c.tension << " " << strength << " " << c.damage << endl;
      //c.disconnect();
      removals.push_back(&c);
    }


    if(c.transient)
    {
//       cerr << "disconnect: " << strength << " " << c.tension << endl;
    }
  }

  for(list<Connection *>::iterator it = removals.begin();
      it != removals.end(); it++)
  {
    Connection &c = **it;

    physical.removeConnection(c);
  }
}



bool IntersectionDetector::isIntersection(Element &e1, Element &e2)
{
   double rnorm;

   Vector r(3);

   r = e1.position;
   r.axpy(-1, e2.position);
   rnorm = r.norm();

   if(rnorm <= (e1.radius + e2.radius))
   {
     return true;
   }

   return false;
 }

}
