#if !defined(PHYSICALDOLFININTERFACE_HPP)
#define PHYSICALDOLFININTERFACE_HPP

#include <iostream>
#include <vector>

#include <dolfin.h>

#include "ko/LinAlg.hpp"
#include "ko/Element.hpp"
#include "ko/Connection.hpp"
#include "ko/Material.hpp"
#include "ko/Physical.hpp"
#include "ko/MechanicsModel.hpp"

namespace ko
{

class PhysicalDolfinInterface
{
public:
  PhysicalDolfinInterface(dolfin::Mesh& mesh, Physical& physical,
			  dolfin::Function& velocity, dolfin::real young,
			  dolfin::real damp);

  void updatePhysical();
  void build();
  void addVertex(dolfin::Vertex& vertex);
  void addCell(dolfin::Cell& cell);
  void addBoundaryCell(dolfin::Cell& cell);
  void updateVertex(dolfin::Vertex& iv, dolfin::Vertex& bv);
  void updateMaterial();

  void computefcontact(dolfin::Vector& fcontact, MechanicsModel& mmodel);

  dolfin::Mesh& mesh;
  dolfin::Mesh* boundary;

  dolfin::MeshFunction<uint> vertex_map;
  dolfin::MeshFunction<uint> cell_map;

  dolfin::Function& velocity;
  Physical& physical;

  Material m;

  dolfin::real young;
  dolfin::real damp;

  std::vector<Element *> elements;
  std::vector<dolfin::Vertex *> vertices;
};

}

#endif // PHYSICAL_HPP
