%module(directors="1") ko


%{
#include <dolfin.h>
#include <dolfin/constants.h>
#include <ko/Element.hpp>
#include <ko/Connection.hpp>
#include <ko/Physical.hpp>
#include <ko/Simulator.hpp>
#include <ko/PhysicalDolfinInterface.hpp>
#include <ko/PhysicalDOMTranslator.hpp>
#include <ko/IntersectionDetector.hpp>

#include <string>
  
using namespace ko;
%}

%typemap(python,in) dolfin::real = double; 
%typemap(python,out) dolfin::real = double; 
%typemap(python,in) dolfin::uint = int; 
%typemap(python,out) dolfin::uint = int; 

// Typemaps for dolfin::real array arguments in virtual methods
// probably not very safe
%typemap(directorin) dolfin::real [] {
  {
    // Custom typemap
    $input = SWIG_NewPointerObj((void *) $1_name, $1_descriptor, $owner);
  }
}

%typemap(directorin) dolfin::real const [] {
  {
    // Custom typemap
    $input = SWIG_NewPointerObj((void *) $1_name, $1_descriptor, $owner);
  }
}


%include "typemaps.i"
%include "std_string.i"
%include "std_list.i"

%include "carrays.i"

%array_functions(dolfin::real, realArray);

%ignore ko::print;

%template(ElementList) std::list<ko::Element *>;
%template(ConnectionList) std::list<ko::Connection *>;

%rename(Vector) ko::SimpleVector;

%import "dolfin/GenericVector.h"

// Declare dummy uBlas classes

%inline %{
  namespace boost{ namespace numeric{ namespace ublas{}}}
%}

namespace dolfin {
  class ublas_dense_matrix {};
  class ublas_sparse_matrix {};
}

%import "dolfin/Vector.h"
%import "dolfin/GenericVector.h"

%import "dolfin/SparseVector.h"
%import "dolfin/DenseVector.h"
%import "dolfin/uBlasVector.h"

#ifdef HAVE_PETSC_H
%import "dolfin/PETScVector.h"
#endif

/* Ko public interface */

/* physical includes */

%include "ko/LinAlg.hpp"
%include "ko/SimpleVector.hpp"
%include "ko/Element.hpp"
%include "ko/Material.hpp"
%include "ko/Connection.hpp"
%include "ko/Physical.hpp"
%include "ko/MechanicsModel.hpp"
%include "ko/BSPTree.hpp"
//%include <ko/Simulator.hpp>
%include "ko/IntersectionDetector.hpp"
%include "ko/PhysicalDolfinInterface.hpp"
%include "ko/PhysicalDOMTranslator.hpp"

