import sys
import datetime
import Numeric
import LinearAlgebra
from dolfin import *
import ko
import transform
from kocontact import *

class DiffPDE(TimeDependentPDE):
    def __init__(self, mesh, envp, f, E, nu, nuv, nuplast, yld,
                 bc, k, T, v0, r):

        self.U = Function(Vector(), mesh)
        self.V = Function(Vector(), mesh)
        self.B = Function(Vector(), mesh)
        self.F = Function(Vector(), mesh)
        self.Sigma = Function(Vector(), mesh)
        self.Epsilon = Function(Vector(), mesh)
        self.SigmaNorm = Function(Vector(), mesh)

        self.lmbda = E * nu / ((1 + nu) * (1 - 2 * nu))
        self.mu = E / (2 * (1 + nu))

        self.do_plast = True
        #self.yld = 1.0e1
        #self.nuplast = 1e-02
        #self.nuplast = 1.0e2

        self.nuplast = nuplast
        self.yld = yld

        #direct_forms = import_formfile("ElasticityDirect.form")
        import elasticitydirectform as direct_forms

        #mass_forms = import_formfile("ElasticityMass.form")
        import elasticitymassform as mass_forms

        #load_forms = import_formfile("ElasticityLoad.form")
        import elasticityloadform as load_forms

        #f_forms = import_formfile("ElasticityUpdatedDeformation.form")
        import elasticityupdateddeformationform as f_forms

        #elast_forms = import_formfile("ElasticityUpdatedEquilibrium.form")
        import elasticityupdatedequilibriumform as elast_forms

        #stress_forms = import_formfile("ElasticityUpdatedStress.form")
        import elasticityupdatedstressform as stress_forms


        self.amass = mass_forms.ElasticityMassBilinearForm()
        self.Ldirect = direct_forms.ElasticityDirectLinearForm(self.B, self.V,
                                                               r, self.lmbda,
                                                               self.mu, nuv)

        self.Ldiff = elast_forms.\
                     ElasticityUpdatedEquilibriumLinearForm(self.Sigma,
                                                            self.Epsilon,
                                                            self.V, r, nuv)
        
        self.LF = f_forms.\
                  ElasticityUpdatedDeformationLinearForm(self.V, self.F)
        self.Lstress = stress_forms.\
                       ElasticityUpdatedStressLinearForm(self.V, self.Sigma,
                                                         self.B,
                                                         self.SigmaNorm,
                                                         self.lmbda, self.mu,
                                                         self.nuplast)

        Lload = load_forms.ElasticityLoadLinearForm(f)


        self.U.init(mesh, self.amass.trial())
        self.V.init(mesh, self.amass.trial())
        self.B.init(mesh, self.Ldirect.element(0))
        self.F.init(mesh, self.LF.element(1))
        self.Sigma.init(mesh, self.Ldiff.element(0))
        self.Epsilon.init(mesh, self.Ldiff.element(1))
        self.SigmaNorm.init(mesh, self.Lstress.element(3))

        N = self.U.vector().size() + self.V.vector().size() + \
            self.F.vector().size() + self.Sigma.vector().size()

        TimeDependentPDE.__init__(self, self.amass, self.Ldiff,
                                  mesh, bc, N, k, T)

        self.v0 = v0
        self.r = r
        
        self.E = E
        self.nu = nu
        self.nuv = nuv
        
        self.solutionfile = File("solution.pvd")
        self.counter = 0
        self.filecounter = 0
        self.fcount = 0

        self.envp = envp
        



        self.xu = self.U.vector()
        self.xv = self.V.vector()
        self.xB = self.B.vector()
        self.xF = self.F.vector()
        self.xSigma = self.Sigma.vector()
        self.xEpsilon = self.Epsilon.vector()
        self.xSigmaNorm = self.SigmaNorm.vector()

    
        self.dotxu = Vector(self.xu)
        self.dotxv = Vector(self.xv)
        self.dotxF = Vector(self.xF)
        self.dotxSigma = Vector(self.xSigma)
        
        self.M = Matrix()
        self.m = Vector()
        self.msigma = Vector(self.xB)
        
        self.bload = Vector()
        self.fcontact = Vector(self.xu)
    
        #self.xF = Vector(self.xB)
        self.xF0 = Vector(self.xB)
        self.xF1 = Vector(self.xB)
        
        self.dotxu_sc = Vector_createScatterer(self.dotxu, self.dotx,
                                               0, self.dotxu.size())
        self.dotxv_sc = Vector_createScatterer(self.dotxv, self.dotx,
                                               self.dotxv.size(),
                                               self.dotxv.size())
        self.dotxF_sc = Vector_createScatterer(self.dotxF, self.dotx,
                                               self.dotxu.size() +
                                               self.dotxv.size(),
                                               self.dotxF.size())
        self.dotxSigma_sc = Vector_createScatterer(self.dotxSigma, self.dotx,
                                                   self.dotxu.size() +
                                                   self.dotxv.size() +
                                                   self.dotxF.size(),
                                                   self.dotxSigma.size())

        # Initialize and compute coefficients

        # Mass matrix
        assemble(self.a(), self.M, self.mesh())
        FEM_lump(self.M, self.m)

        print "m:"
        self.m.disp()

        # U
        ElasticityUpdatedSolver_initu0(self.xu, self.U.element(), self.mesh())

        # V
        ElasticityUpdatedSolver_finterpolate(self.v0, self.V, self.mesh())
        dolfin_log(False)
        #applyBC(self.xv, self.mesh(), self.V.element(), self.bc())
        dolfin_log(True)

        #print "xv:"
        #self.xv.disp()

        # B
        ElasticityUpdatedSolver_initmsigma(self.msigma, self.B.element(),
                                           self.mesh())
        ElasticityUpdatedSolver_initF0Euler(self.xF0, self.B.element(),
                                            self.mesh())
        ElasticityUpdatedSolver_computeFBEuler(self.xF, self.xB, self.xF0,
                                               self.xF1, self.B.element(),
                                               self.mesh())
        ElasticityUpdatedSolver_computeBEuler(self.xF, self.xB,
                                              self.F.element(),
                                              self.mesh())

        dolfin_log(False)
        assemble(Lload, self.bload, self.mesh())
        #applyBC(self.bload, self.mesh(), self.a().trial(), self.bc())
        dolfin_log(True)

        print "bload:"
        self.bload.disp()

        # Initial values for ODE
        # Gather into x
        Vector_gather(self.xu, self.x, self.dotxu_sc)
        Vector_gather(self.xv, self.x, self.dotxv_sc)
        Vector_gather(self.xF, self.x, self.dotxF_sc)
        Vector_gather(self.xSigma, self.x, self.dotxSigma_sc)

        # Mass-spring contact model

        self.kocontact = KoContactModel(mesh, envp, self.V, E, nu, nuv)

        self.kocontact.interface.m.damp *= 100.0

    def init(self, U):
        print "Python init"
        
    def preparestep(self):
        self.kocontact.computecontacts()

    def prepareiteration(self):
        self.kocontact.computeforce(self.fcontact)

        #print "fcontact:"
        #self.fcontact.disp()

    def fu(self, x, dotx, t):
        #print "Python fu"

        Vector_scatter(self.xu, self.x, self.dotxu_sc)
        Vector_scatter(self.xv, self.x, self.dotxv_sc)
        Vector_scatter(self.xF, self.x, self.dotxF_sc)
        Vector_scatter(self.xSigma, self.x, self.dotxSigma_sc)

        # Mesh

        ElasticityUpdatedSolver_deform(self.mesh(), self.U)

        ElasticityUpdatedSolver_computeBEuler(self.xF, self.xB,
                                              self.F.element(),
                                              self.mesh())

        # Mass matrix

        ElasticityUpdatedSolver_initmsigma(self.msigma, self.B.element(),
                                           self.mesh())

        # Sigma

        if(self.do_plast):
            ElasticityUpdatedSolver_plasticity(self.xSigma,
                                               self.xSigmaNorm,
                                               self.yld,
                                               self.Lstress.element(3),
                                               self.mesh())

            #self.xSigmaNorm.disp()

        dolfin_log(False)
        assemble(self.Lstress, self.dotxSigma, self.mesh())
        dolfin_log(True)
        
        self.dotxSigma.div(self.msigma)

        self.xEpsilon.copy(self.dotxSigma)
        self.xEpsilon *= 1.0 / self.lmbda

        # U
        
        self.dotxu.copy(self.xv)

        # V

        dolfin_log(False)
        assemble(self.L(), self.dotxv, self.mesh())
        #applyBC(self.dotxv, self.mesh(), self.V.element(), self.bc())
        dolfin_log(True)
    
        #if(t > 1.0 and t < 3.0):
        self.dotxv += self.bload
        
        self.dotxv.div(self.m)

        self.dotxv += self.fcontact

        # F

        dolfin_log(False)
        assemble(self.LF, self.dotxF, self.mesh())
        dolfin_log(True)

        self.dotxF.div(self.msigma)

        # Gather into dotx
        Vector_gather(self.dotxu, self.dotx, self.dotxu_sc)
        Vector_gather(self.dotxv, self.dotx, self.dotxv_sc)
        Vector_gather(self.dotxF, self.dotx, self.dotxF_sc)
        Vector_gather(self.dotxSigma, self.dotx, self.dotxSigma_sc)

        self.fcount += 1

    def save(self, U, t):

        if((self.counter % (33 * 1 * 2)) == 0):
            print "t: ", t
            self.solutionfile << U
            self.kocontact.save("%6.6d.xml" % self.filecounter)
            #print "fcontact:"
            #self.fcontact.disp()
            #print "bload:"
            #self.bload.disp()
            self.filecounter += 1

        self.counter += 1
