import sys
import datetime
import Numeric
import LinearAlgebra
from dolfin import *
import ko
import transform
from kocontact import *
#from korenderer import *

class DirectPDE(TimeDependentPDE):
    def __init__(self, mesh, envp, f, fstress, E, nu, nuvval,
                 bc, T, v0, r, rho):

        self.U = Function(Vector(), mesh)
        self.V = Function(Vector(), mesh)
        self.B = Function(Vector(), mesh)

        lmbdaval = E * nu / ((1 + nu) * (1 - 2 * nu))
        muval = E / (2 * (1 + nu))

        self.lmbda = Function(lmbdaval)
        self.mu = Function(muval)
        self.nuv = Function(nuvval)

        direct_forms = import_formfile("ElasticityDirect.form")
        #import elasticitydirectform as direct_forms

        mass_forms = import_formfile("ElasticityMass.form")
        #import elasticitymassform as mass_forms

        load_forms = import_formfile("ElasticityLoad.form")
        #import elasticityloadform as load_forms

        stressload_forms = import_formfile("ElasticityStressLoad.form")
        #import elasticitystressloadform as stressload_forms

        visco_forms = import_formfile("ElasticityVisco.form")
        #import elasticityviscoform as visco_forms

        self.amass = mass_forms.ElasticityMassBilinearForm(rho)

        self.Ldirect = direct_forms.ElasticityDirectLinearForm(self.B, self.V,
                                                               self.lmbda,
                                                               self.mu,
                                                               self.nuv)
        #self.Ldirect = direct_forms.ElasticityDirectLinearForm(self.B,
        #                                                       lmbda,
        #                                                       mu)

        avisco = visco_forms.ElasticityViscoBilinearForm(self.nuv)
        
        Lload = load_forms.ElasticityLoadLinearForm(f)
        self.Lsload = stressload_forms.ElasticityStressLoadLinearForm(fstress)

        self.U.init(mesh, self.amass.trial())
        self.V.init(mesh, self.amass.trial())
        self.B.init(mesh, self.Ldirect.element(0))

        N = self.U.vector().size() + self.V.vector().size()

        TimeDependentPDE.__init__(self, self.amass, self.Ldirect,
                                  mesh, bc, N, T)

        self.v0 = v0
        self.r = r
        
        self.E = E
        self.nu = nu
        #self.nuv = nuv
        
        self.solutionfile = File("solution.pvd")
        self.counter = 0
        self.filecounter = 0
        self.fcount = 0

        self.envp = envp
        


        self.xu = self.U.vector()
        self.xv = self.V.vector()
        self.xB = self.B.vector()
    
        self.dotxu = Vector(self.xu.size())
        self.dotxv = Vector(self.xv.size())
        
        self.M = Matrix()
        self.Epsilon = Matrix()
        self.m = Vector()
        self.msigma = Vector(self.xB.size())
        
        self.bload = Vector()
        self.bsload = Vector()
        self.fcontact = Vector(self.xu.size())
        self.xtmp = Vector(self.xu.size())
    
        self.xF = Vector(self.xB.size())
        self.xF0 = Vector(self.xB.size())
        self.xF1 = Vector(self.xB.size())
        
        #self.dotxu_sc = Vector_createScatterer(self.dotxu, self.dotx,
        #                                       0, self.dotxu.size())
        #self.dotxv_sc = Vector_createScatterer(self.dotxv, self.dotx,
        #                                       self.dotxv.size(),
        #                                       self.dotxv.size())


        # Initialize and compute coefficients

        # Mass matrix
        FEM_assemble(self.a(), self.M, self.mesh())
        FEM_lump(self.M, self.m)

        print "m:"
        self.m.disp()

        # Epsilon matrix
        FEM_assemble(avisco, self.Epsilon, self.mesh())

        # U
        ElasticityUpdatedSolver_initu0(self.xu, self.U.element(), self.mesh())

        # V
        ElasticityUpdatedSolver_finterpolate(self.v0, self.V, self.mesh())
        dolfin_log(False)
        #applyBC(self.xv, self.mesh(), self.V.element(), self.bc())
        dolfin_log(True)

        #print "xv:"
        #self.xv.disp()

        # B
        ElasticityUpdatedSolver_initmsigma(self.msigma, self.B.element(),
                                           self.mesh())
        ElasticityUpdatedSolver_initF0Euler(self.xF0, self.B.element(),
                                            self.mesh())
        ElasticityUpdatedSolver_computeFBEuler(self.xF, self.xB, self.xF0,
                                               self.xF1, self.B.element(),
                                               self.mesh())

        dolfin_log(False)
        FEM_assemble(Lload, self.bload, self.mesh())
        #applyBC(self.bload, self.mesh(), self.a().trial(), self.bc())
        dolfin_log(True)

        #print "bload:"
        #self.bload.disp()

        dolfin_log(False)
        FEM_assemble(self.Lsload, self.bsload, self.mesh())
        #applyBC(self.bload, self.mesh(), self.a().trial(), self.bc())
        dolfin_log(True)

        # Initial values for ODE
        # Gather into x
        #Vector_gather(self.xu, self.x, self.dotxu_sc)
        #Vector_gather(self.xv, self.x, self.dotxv_sc)
        #self.xu.copy(self.x, 0, 0, self.xu.size())
        #self.xv.copy(self.x, 0, self.xu.size(), self.xv.size())
        self.x.copy(self.xu, 0, 0, self.xu.size())
        self.x.copy(self.xv, self.xu.size(), 0, self.xv.size())

        print "xu:"
        self.xu.disp()

        print "xv:"
        self.xv.disp()


        # Mass-spring contact model
        self.kocontact = KoContactModel(mesh, envp, self.V, E, nu, nuvval)

        # Renderer

        #self.renderer = KoRenderer()

    def init(self, U):
        print "Python init"
        
    def preparestep(self):
        1
        self.kocontact.computecontacts()

    def prepareiteration(self):
        1
        self.kocontact.computeforce(self.fcontact)

        #print "fcontact:"
        #self.fcontact.disp()

    def fu(self, x, dotx, t):
        #print "Python fu"

        #Vector_scatter(self.xu, self.x, self.dotxu_sc)
        #Vector_scatter(self.xv, self.x, self.dotxv_sc)
        self.xu.copy(self.x, 0, 0, self.dotxu.size())
        self.xv.copy(self.x, 0, self.dotxu.size(), self.dotxv.size())

        # Mesh

        ElasticityUpdatedSolver_deform(self.mesh(), self.U)

        ElasticityUpdatedSolver_computeFBEuler(self.xF, self.xB,
                                               self.xF0, self.xF1,
                                               self.B.element(),
                                               self.mesh())


        # U
        
        self.dotxu.copy(self.xv, 0, 0, self.xv.size())

        # V

        dolfin_log(False)
        #assemble(self.Lsload, self.bsload, self.mesh())
        dolfin_log(True)

        dolfin_log(False)
        FEM_assemble(self.L(), self.dotxv, self.mesh())
        #applyBC(self.dotxv, self.mesh(), self.V.element(), self.bc())
        dolfin_log(True)
    
        #self.Epsilon.mult(self.xv, self.xtmp)
        self.dotxv.axpy(1.0, self.xtmp)

        self.dotxv.axpy(1.0, self.bload)

        #if(t > 1.0 and t < 4.0):
        #    self.dotxv += self.bsload

        self.dotxv.div(self.m)

        self.dotxv.axpy(1.0, self.fcontact)

        # Gather into dotx
        #Vector_gather(self.dotxu, self.dotx, self.dotxu_sc)
        #Vector_gather(self.dotxv, self.dotx, self.dotxv_sc)
        self.dotx.copy(self.dotxu, 0, 0, self.dotxu.size())
        self.dotx.copy(self.dotxv, self.dotxu.size(), 0, self.dotxv.size())


        self.fcount += 1

    def save(self, U, t):

        if((self.counter % (33)) == 0):
            print "t: ", t
            self.solutionfile << U
            #self.kocontact.save("%6.6d.xml" % self.filecounter)
            #print "fcontact:"
            #self.fcontact.disp()
            #print "bload:"
            #self.bload.disp()
            self.filecounter += 1

            self.kocontact.interface.updatePhysical()
            #self.renderer.render(self.kocontact.physical, t)



        #if(self.counter == 200):
        #    print "FOOFOO"
        #    self.xF0 *= 1.1


        self.counter += 1
