import sys
import datetime
import Numeric
import LinearAlgebra
from dolfin import *
import ko
import transform
from kocontact import *
from directpde import *

class Source(Function):
    def eval(self, point, i):
        if(i == 1 and point.y > -100.0):
            return -10.0 * 1.0
        else:
            return 0.0

class SimpleBC(BoundaryCondition):
    def eval(self, value, point, i):
        if point.x == 0.0:
            value.set(0.0)

class ZeroVelocity(Function):
    def eval(self, point, i):
        return 0.0


class InitialVelocity(Function):
    def eval(self, point, i):

        center = Point()
        r = Point()
        w = Point()

        center.x = 0.5
        center.y = 0.5
        center.z = 0.25

        r.x = point.x - center.x
        r.y = point.y - center.y
        r.z = point.z - center.z

        w.x = 0.0
        w.y = 0.0
        w.z = 10.0

        v = Point(w.cross(r))

        #print "x: ", r.x
        #print "y: ", r.y
        #print "v.x: ", v.x
        #print "v.y: ", v.y

        result = 0.0

        if(i == 0):
            result += v.x
        if(i == 1):
            result += v.y

        #if(i == 0):
        #    return 1.0
        #else:
        #    return 0.0

        #if(i == 0 and point.y > -2.0):
        #    return 15.0
        #elif(i == 0):
        #    return 10.0
        #elif(i == 1):
        #    return 5.0
        #else:
        #    return 0.0

        if(i == 0 and point.y > 3.7):
            return 0.5

        return 0.0

        #return result * 0.0


E   = 500.0 * 0.5 * 1 # Young's modulus
nu  = 0.3 # Poisson's ratio
nuv = 1.0 * 1.0e1 * 0.2 # Viscosity

mesh = Mesh()

cowmesh0 = Mesh("cow05c.xml")
cowmesh1 = Mesh("cow05c.xml")

A = Numeric.zeros([3, 3], 'd')
A[0, 1] = -1.0
A[1, 0] = 1.0
A[2, 2] = 1.0
b = Numeric.zeros(3, 'd')
b[0] = 2.8
b[1] = 0.7 - 3.2
b[2] = 0.2
transform.transform(cowmesh0, A, b)

A = Numeric.zeros([3, 3], 'd')
A[0, 1] = -1.0
A[1, 0] = 1.0
A[2, 2] = 1.0
b = Numeric.zeros(3, 'd')
b[0] = -1.9
b[1] = 4.1
b[2] = 0.2
transform.transform(cowmesh1, A, b)

#mesh.merge(cowmesh0)
mesh.merge(cowmesh1)


A = Numeric.zeros([3, 3], 'd')
A[0, 2] = 1.5
A[1, 1] = 1.5
A[2, 0] = 1.5

block0 = Mesh("block03a.xml")
#block0 = Mesh("simpletet.xml")
b = Numeric.zeros(3, 'd')
b[0] = -4.1
b[1] = 2.0
b[2] = -0.5
transform.transform(block0, A, b)

block1 = Mesh("block03a.xml")
#block1 = Mesh("simpletet.xml")
b = Numeric.zeros(3, 'd')
b[0] = -4.1 + 1 * 1.5
b[1] = 0.5
b[2] = -0.5
transform.transform(block1, A, b)

block2 = Mesh("block03a.xml")
#block1 = Mesh("simpletet.xml")
b = Numeric.zeros(3, 'd')
b[0] = -4.1 + 2 * 1.5
b[1] = -1.0
b[2] = -0.5
transform.transform(block2, A, b)

block3 = Mesh("block03a.xml")
#block1 = Mesh("simpletet.xml")
b = Numeric.zeros(3, 'd')
b[0] = -4.1 + 3 * 1.5
b[1] = -2.5
b[2] = -0.5
transform.transform(block3, A, b)

block4 = Mesh("block03a.xml")
#block1 = Mesh("simpletet.xml")
b = Numeric.zeros(3, 'd')
b[0] = -4.1 + 4 * 1.5
b[1] = -4.0
b[2] = -0.5
transform.transform(block4, A, b)

envmesh = Mesh()
envmesh.merge(block0)
envmesh.merge(block1)
envmesh.merge(block2)
envmesh.merge(block3)
envmesh.merge(block4)
#mesh.merge(mesh2)

vzero = ZeroVelocity()

envp = ko.Physical()
envi = ko.PhysicalDolfinInterface(envmesh, envp, vzero, E,
                                  nuv, 0)

envi.updatePhysical()

floor0 = ko.Element()
floor0.position.setval(0, 0.0)
floor0.position.setval(1, -10000)
floor0.position.setval(2, 0.0)
floor0.radius = 10000 - 6.0
#floor0.material = envi.m
envp.addElement(floor0)

floor1 = ko.Element()
floor1.position.setval(0, 10000.0)
floor1.position.setval(1, 0.0)
floor1.position.setval(2, 0.0)
floor1.radius = 10000 - 6.0
#floor1.material = envi.m
envp.addElement(floor1)

floor2 = ko.Element()
floor2.position.setval(0, -10000.0)
floor2.position.setval(1, 0.0)
floor2.position.setval(2, 0.0)
floor2.radius = 10000 - 6.0
#floor2.material = envi.m
envp.addElement(floor2)

floor3 = ko.Element()
floor3.position.setval(0, 0.0)
floor3.position.setval(1, 0.0)
floor3.position.setval(2, 10000.0)
floor3.radius = 10000 - 6.0
#floor3.material = envi.m
envp.addElement(floor3)

floor4 = ko.Element()
floor4.position.setval(0, 0.0)
floor4.position.setval(1, 0.0)
floor4.position.setval(2, -10000.0)
floor4.radius = 10000 - 6.0
#floor4.material = envi.m
envp.addElement(floor4)

T = 6.0
#T = 1.0e-4 / 1.0
#k = 1.0e-3 / 4.0
#k = 1.0e-3
k = 1.0e-3 / 2.0


load_parameters("parameters.xml")

# Coefficients

f = Source()
v0 = InitialVelocity()
r = Resistance()
bc = UtilBC1()
        

dpde = DirectPDE(mesh, envp, f, E, nu, nuv,
                 bc, k, T, v0, r)

dpde.solve(dpde.U)

print "fcount: ", dpde.fcount
