import LinearAlgebra
from dolfin import *
import ko

class KoContactModel:
    def __init__(self, mesh, envp, V, E, nu, nuv):

        # Mass-spring contact model

        self.physical = ko.Physical()
        self.mmodel = ko.MechanicsModel()
        self.interface = ko.PhysicalDolfinInterface(mesh,
                                                    self.physical,
                                                    V, E, nuv)
        self.idetector = ko.IntersectionDetector()
        self.domtranslator = ko.PhysicalDOMTranslator()

        self.idetector.init(self.physical.elements)

        self.interface.m.density = 1.0
        self.interface.m.friction = 0.3
        self.interface.m.young = 0.3 * E
        self.interface.m.damp = 1.0 * nuv

        pi = envp.elements.begin()
        pend = envp.elements.end()
        while(pi != pend):
            e = pi.value()
            e.artificial = True
            e.material = self.interface.m
            pi.next()

        self.physical.merge(envp)

        self.interface.updatePhysical()

        ko.generateNames(self.physical)

    def computecontacts(self):
        self.idetector.collisions(self.physical)
        self.idetector.fractures(self.physical)

    def computeforce(self, fcontact):
        self.interface.computefcontact(fcontact, self.mmodel)

    def save(self, filename):
        self.domtranslator.writefile(filename, self.physical)
