import LinearAlgebra
from dolfin import *
import ko

from OpenGL.GL import *
from OpenGL.GLU import *
import pygame
from pygame.locals import *
import time


class KoRenderer:
    def __init__(self):

        self.freq = 33.0
        self.period = 1.0 / self.freq
        self.counter = 0

        self.video_flags = OPENGL|DOUBLEBUF

        self.width = 400
        self.height = 400
    
        pygame.init()
        pygame.display.set_mode((self.width, self.height), self.video_flags)

        self.resize((self.width, self.height))
        self.init()

        self.frames = 0
        self.ticks = pygame.time.get_ticks()

        self.rtri = 0.0
        self.rquad = 0.0

    def resize(self, (width, height)):
        if height==0:
            height=1
        glViewport(0, 0, width, height)
        glMatrixMode(GL_PROJECTION)
        glLoadIdentity()
        gluPerspective(105, 1.0*width/height, 0.1, 100.0)
        glMatrixMode(GL_MODELVIEW)
        glLoadIdentity()

        gluLookAt(0.5, 3.0, 5.5,
                  0.0, -1.0, 0.0,
                  0.0, 1.0, 0.0)
        
    def init(self):
        glShadeModel(GL_SMOOTH)
        glClearColor(0.0, 0.0, 0.0, 0.0)
        glClearDepth(1.0)
        glEnable(GL_DEPTH_TEST)
        glDepthFunc(GL_LEQUAL)
        glHint(GL_PERSPECTIVE_CORRECTION_HINT, GL_NICEST)

        glEnable(GL_LIGHTING)
        glShadeModel(GL_SMOOTH)
        glPolygonMode(GL_FRONT_AND_BACK, GL_FILL)
    
        White  = ( (1.0, 1.0, 1.0, 1.0) )
        Black  = ( (0.0, 0.0, 0.0, 1.0) )
        Grey  = ( (0.5, 0.5, 0.5, 1.0) )
        Red  = ( (1.0, 0.0, 0.0, 1.0) )


        glMaterialfv(GL_FRONT, GL_DIFFUSE, Grey)
        glMaterialfv(GL_FRONT, GL_SPECULAR, White)
        glMaterialfv(GL_FRONT, GL_AMBIENT, Grey)
        glMaterialf(GL_FRONT, GL_SHININESS, 20.0)
    
        glMaterialfv(GL_BACK, GL_DIFFUSE, Red)
        glMaterialfv(GL_BACK, GL_SPECULAR, White)
        glMaterialfv(GL_BACK, GL_AMBIENT, Red)
        glMaterialf(GL_BACK, GL_SHININESS, 20.0)

        LightPosition = ( (0.0, 0.0, 2.0, 1.0) )
        LightAmbient  = ( (0.5, 0.5, 0.5, 1.0) )
        LightDiffuse  = ( (1.0, 1.0, 1.0, 1.0) )

        glLightfv( GL_LIGHT1, GL_AMBIENT, LightAmbient )
        glLightfv( GL_LIGHT1, GL_DIFFUSE, LightDiffuse )
        glLightfv( GL_LIGHT1, GL_POSITION, LightPosition )
        glEnable( GL_LIGHT1 )
                




    def draw(self, p):
        glClear(GL_COLOR_BUFFER_BIT|GL_DEPTH_BUFFER_BIT)
        #glLoadIdentity()



        #glTranslatef(0.0, 0.0, -6.0)

        quad = gluNewQuadric()
        
        it = p.elements.begin()
        end = p.elements.end()
        while(it != end):
            e = it.value()

            if(e.radius < 1.0e3):

                glPushMatrix()

                glTranslatef(
                    e.position.getval(0),
                    e.position.getval(1),
                    e.position.getval(2))
        
                gluSphere(quad, e.radius, 8, 8 )
                glPopMatrix()

            it.incr()

        
        
        
        #glLoadIdentity()
        #glTranslatef(-1.5, 0.0, -6.0)

        #glRotatef(self.rtri, 0.0, 1.0, 0.0)

        #glBegin(GL_TRIANGLES)
        #glColor3f(1.0, 0.0, 0.0)
        #glVertex3f(0.0, 1.0, 0.0)
        #glColor3f(0.0, 1.0, 0.0)
        #glVertex3f(-1.0, -1.0, 0)
        #glColor3f(0.0, 0.0, 1.0)
        #glVertex3f(1.0, -1.0, 0)
        #glEnd()


    def events(self):


        event = pygame.event.poll()
        print event
        if event.type == QUIT or \
               (event.type == KEYDOWN and event.key == K_ESCAPE):
            print "Quitting"
        
    def render(self, p, t):

        if(t >= self.counter * self.period):

            print "Rendering"
            self.counter += 1

            self.rtri += 0.2 * 100
            self.rquad+= 0.2 * 100

            self.events()
            self.draw(p)
            pygame.display.flip()
