from dolfin import *
import ko
from directpde import *

class StressSource(Function):
    def eval(self, point, i):
        print "point: ", point[0], " ", point[1], " ", point[2]

        result = 0.0

        if(i == 0 and point[0] > 2.0):
            result -= 50.0


        if(i == 0):
            result -= 50.0 * 0.0

        if(i == 5):
            return 0.0

        if(i == 4):
            return 0.0

        return result

class Source(Function):
    def eval(self, point, i):
        if(i == 1 and point[1] > -100.0):
            return -10.0 * 1.0 * 1.0
        else:
            return 0.0

class Density(Function):
    def eval(self, point, i):
        return 3.0e0

class SimpleBC(BoundaryCondition):
    def eval(self, value, point, i):
        if point[0] == 0.0:
            value.set(0.0)

class InitialVelocity(Function):
    def eval(self, point, i):
        return 0.0


set("ODE method", "dg");
set("ODE order", 0);
set("ODE nonlinear solver", "fixed-point");
set("ODE linear solver", "direct");
set("ODE tolerance", 1.0e3);
set("ODE discrete tolerance", 1.0e3);

set("ODE fixed time step", True);
set("ODE initial time step", 1.0e-2);
set("ODE maximum time step", 1.0e-2);

set("ODE save solution", False);
set("ODE solution file name", "primal.py");
set("ODE number of samples", 100);


nu  = 0.3 # Poisson's ratio
E   = 500.0 * 0.7 * 1.0 # Young's modulus
nuv = 1.0 * 1.0e2 * 1.0e-3 * 100.0 # Viscosity

yld = 4.0e3 * 1.0e-3 # Yield strength
nuplast = 1.0e-4 * 1.0e3 * 4.0e-1 # Plastic viscosity

import geometry

T = 10.0
#T = 1.0e-4 / 1.0
#k = 1.0e-3 / 4.0
#k = 1.0e-3 * 3.333
k = 3.333e-3 * 3


#load_parameters("parameters.xml")

# Coefficients

f = Source()
fstress = StressSource()
rho = Density()
v0 = InitialVelocity()
r = Resistance()
bc = UtilBC1()
        

pde = DirectPDE(geometry.mesh, geometry.envp, f, fstress, E, nu, nuv, bc, T,
                v0, r, rho)

pde.solve(pde.U)

print "fcount: ", pde.fcount

