#include <iostream>
#include <strstream>
#include <fstream>

#include <dolfin.h>

#include "ko/PhysicalDOMTranslator.hpp"
#include "ko/Physical.hpp"
#include "ko/Element.hpp"

using namespace ko;
using namespace std;
using namespace xercesc;

void addVertex(dolfin::Vertex &vertex, Physical &p, Material &m,
	     std::vector<Element *> &elements)
{
  Element &e = *(new Element());

  // Determine radius
  // 0.3 * max of neighbor cell diameter

  dolfin::real maxr = 0.0;
  for(dolfin::CellIterator cell(vertex); !cell.end(); ++cell)
  {
    dolfin::Cell &c = *cell;
    maxr = std::max(maxr, c.diameter());
  }

  maxr *= 0.3;


//   e.radius = 0.16;
  e.radius = maxr;

  e.position(0) = vertex.point().x();
  e.position(1) = vertex.point().y();
  e.position(2) = vertex.point().z();

  e.material = &m;

  std::ostrstream name;
  name << vertex.index() << ends;
  
  e.name = name.str();

  p.elements.push_back(&e);
  elements[vertex.index()] = &e;
}

void addCell(dolfin::Cell &cell, Physical &p, 
	     std::vector<Element *> &elements)
{
  for(int i = 0; i < 4; i++)
  {
    for(int j = 0; j < 4; j++)
    {
      Element &e0 = *(elements[cell.entities(0)[i]]);
      Element &e1 = *(elements[cell.entities(0)[j]]);

      if((i != j) && !e0.isConnected(e1))
      {
	Connection &c = *(new Connection());
	
	Vector r(3);
	double rnorm;
	
	r = e0.position;
	r.axpy(-1, e1.position);
	
	rnorm = r.norm();
	
	c.connect(e0, e1);
	c.length = rnorm;
	
	p.connections.push_back(&c);
      }
    }
  }
}

int main(int argC, char* argV[])
{
  dolfin::dolfin_output("silent");


  if(argC > 1)
  {
    dolfin::Mesh mesh(argV[1]);
    
    Physical p;
    PhysicalDOMTranslator t;
    
    Material &m = *(new Material());
    
    m.density = 8E3;
    m.young = 7E9;
    m.damp = 1E2;
    m.strength = 1E8;
    
    m.name = "material";
    
    p.materials.push_back(&m);
    
    std::vector<Element *> elements(mesh.numVertices());

    for(dolfin::VertexIterator n(mesh); !n.end(); ++n)
    {
      dolfin::Vertex &vertex = *n;

      addVertex(vertex, p, m, elements);
    }

    for(dolfin::CellIterator c(mesh); !c.end(); ++c)
    {
      dolfin::Cell &cell = *c;

      addCell(cell, p, elements);
    }


    // Connect nearby elements together (a tetrahedral pattern is too sparse)
    for(list<Element *>::iterator it = p.elements.begin();
	it != p.elements.end(); it++)
    {
      Element &e = **it;

      for(list<Connection *>::iterator it2 = e.connections.begin();
	  it2 != e.connections.end(); it2++)
      {
	Connection &ce = **it2;

	Element *etmp = 0;
	
	if(ce.element1 == &e)
	{
	  etmp = ce.element2;
	}
	else
	{
	  etmp = ce.element1;
	}

	for(list<Connection *>::iterator it3 = etmp->connections.begin();
	    it3 != etmp->connections.end(); it3++)
	{
	  Connection &ce = **it3;
	  
	  Element *e2 = 0;
	  
	  if(ce.element1 == &e)
	  {
	    e2 = ce.element2;
	  }
	  else
	  {
	    e2 = ce.element1;
	  }
	  
	  Vector r(3);
	  r = e.position;
	  r.axpy(-1, e2->position);
	  
	  dolfin::real distance;
	  
	  distance = r.norm();
	  
	  if(distance <= 3.5 * e.radius)
	  {
	    if(!e.isConnected(*e2))
	    {
	      Connection &c = *(new Connection());
	      
	      c.connect(e, *e2);
	      c.length = distance;
	      p.addConnection(c);
	      
// 	      std::cerr << "added connection" << std::endl;
	    }
	  }
	}
      }
    }

    if(argC > 2)
    {
      ofstream os;
      os.open(argV[2], ios::out);
      
      t.write(os, p);
    }
    else
    {
      ofstream os;
      os.open("ko-mesh.xml", ios::out);
      
      t.write(os, p);
    }
  }      


  return 0;
}

